from types import TracebackType
from typing import Any, Literal, Optional

from aiohttp import ClientSession


class PixivHttpClient:
    API_URL = "https://www.pixiv.net"

    def __init__(
        self, session: Optional[ClientSession] = None, lang: str = "ko"
    ) -> None:
        self.session = session
        self.lang = lang

    @property
    def headers(self) -> dict[str, str]:
        return {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36",
            "Referer": self.API_URL,
        }

    @headers.setter
    def headers(self, value: dict[str, str]) -> None:
        self.headers.update(value)

    @property
    def params(self) -> dict[str, Any]:
        return {"lang": self.lang}

    @params.setter
    def params(self, value: dict[str, Any]) -> None:
        self.params.update(value)

    async def close(self):
        if self.session:
            await self.session.close()

    async def __aenter__(self):
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ):
        await self.close()

    async def request(
        self, method: Literal["GET", "POST"], path: str, **kwargs: Any
    ) -> Any:
        if not self.session:
            self.session = ClientSession(headers=self.headers)

        async with self.session.request(
            method, f"{self.API_URL}/ajax/{path}", **kwargs
        ) as r:
            return await r.json()

    async def get_userinfo(self, user_id: int) -> Any:
        params = {"full": 0, **self.params}
        return await self.request("GET", f"user/{user_id}", params=params)

    async def get_full_userinfo(self, user_id: int) -> Any:
        params = {"full": 1, **self.params}
        return await self.request("GET", f"user/{user_id}", params=params)

    async def get_illust(self, illust_id: int) -> Any:
        params = {**self.params}
        return await self.request("GET", f"illust/{illust_id}", params=params)

    async def get_ranking(
        self,
        mode: Literal["daily", "weekly", "monthly"] = "daily",
        date: Optional[str] = None,
    ) -> Any:
        params = {"format": "json", "mode": mode}
        if date:
            params.update({"date": date})

        return await self.request("GET", "ranking", params=params)



from pypixiv.ranking.item import IllustContentType
from typing import Any


class ContentBody:
    def __init__(self, response: Any) -> None:
        self._response = response

    @property
    def title(self) -> str:
        return self._response["title"]

    @property
    def date(self) -> str:
        return self._response["date"]

    @property
    def tags(self) -> list[str]:
        return self._response["tags"]

    @property
    def url(self) -> str:
        return self._response["url"]

    @property
    def illust_type(self) -> str:
        return self._response["illust_type"]

    @property
    def illust_book_style(self) -> str:
        return self._response["illust_book_style"]

    @property
    def illust_page_count(self) -> int:
        return self._response["illust_page_count"]

    @property
    def user_name(self) -> str:
        return self._response["user_name"]

    @property
    def profile_img(self) -> str:
        return self._response["profile_img"]

    @property
    def illust_content_type(self) -> IllustContentType:
        return IllustContentType(self._response["illust_content_type"])

    @property
    def illust_series(self) -> bool:
        return self._response["illust_series"]

    @property
    def illust_id(self) -> int:
        return self._response["illust_id"]

    @property
    def width(self) -> int:
        return self._response["width"]

    @property
    def height(self) -> int:
        return self._response["height"]

    @property
    def user_id(self) -> int:
        return self._response["user_id"]

    @property
    def rank(self) -> int:
        return self._response["rank"]

    @property
    def yes_rank(self) -> int:
        return self._response["yes_rank"]

    @property
    def rating_count(self) -> int:
        return self._response["rating_count"]

    @property
    def view_count(self) -> int:
        return self._response["view_count"]

    @property
    def illust_upload_timestamp(self) -> int:
        return self._response["illust_upload_timestamp"]

    @property
    def attr(self) -> str:
        return self._response["attr"]

    @property
    def is_bookmarked(self) -> bool:
        return self._response["is_bookmarked"]

    @property
    def bookmarkable(self) -> bool:
        return self._response["bookmarkable"]



from pypixiv.user.item import Group, Privacy, SketchLives, WorkSpace
from typing import Any, Optional


class UserInfoBody:
    def __init__(self, response: Any) -> None:
        self._response = response

    @property
    def acceptRequest(self) -> bool:
        return self._response["acceptRequest"]

    @property
    def background(self) -> Optional[str]:
        return self._response["background"]

    @property
    def image(self) -> Optional[str]:
        return self._response["image"]

    @property
    def imageBig(self) -> Optional[str]:
        return self._response["imageBig"]

    @property
    def isBlocking(self) -> bool:
        return self._response["isBlocking"]

    @property
    def isFollowed(self) -> bool:
        return self._response["isFollowed"]

    @property
    def isMypixiv(self) -> bool:
        return self._response["isMypixiv"]

    @property
    def name(self) -> Optional[str]:
        return self._response["name"]

    @property
    def partial(self) -> int:
        return self._response["partial"]

    @property
    def premium(self) -> bool:
        return self._response["premium"]

    @property
    def sketchLiveId(self) -> Optional[str]:
        return self._response["sketchLiveId"]

    @property
    def sketchLives(self) -> list[SketchLives]:
        return [SketchLives(item) for item in self._response["sketchLives"]]


class UserFullInfoBody(UserInfoBody):
    def __init__(self, response: Any) -> None:
        super().__init__(response)

    @property
    def birthDay(self) -> Privacy:
        return Privacy(self._response["birthDay"])

    @property
    def userId(self) -> str:
        return self._response["userId"]

    @property
    def commment(self) -> Optional[str]:
        return self._response["comment"]

    @property
    def commentHtml(self) -> Optional[str]:
        return self._response["commentHtml"]

    @property
    def followedBack(self) -> bool:
        return self._response["followedBack"]

    @property
    def following(self) -> int:
        return self._response["following"]

    @property
    def gender(self) -> Privacy:
        return Privacy(self._response["gender"])

    @property
    def group(self) -> Optional[list[Group]]:
        if group := self._response.get("group"):
            return [Group(item) for item in group]

    @property
    def job(self) -> Privacy:
        return Privacy(self._response["job"])

    @property
    def official(self) -> bool:
        return self._response["official"]

    @property
    def region(self) -> Privacy:
        return Privacy(self._response["region"])

    @property
    def workspace(self) -> WorkSpace:
        return WorkSpace(self._response["workspace"])



from typing import TypedDict

import pytest

from pypixiv.client import PixivClient


class Filterd(TypedDict):
    obj: object
    name: str


def filter_magicmethod(obj: object) -> list[Filterd]:
    return [
        {"obj": obj, "name": property_name}
        for property_name in list(filter(lambda d: "_" not in d, dir(obj)))
    ]


@pytest.mark.asyncio
async def test_userinfo(client: PixivClient):
    user = await client.illustinfo(79136250)
    property_names = filter_magicmethod(user)

    for i in property_names:
        getted = getattr(i["obj"], i["name"])
        if not type(getted).__module__ in ["__builtin__", "builtins"]:
            if son := filter_magicmethod(getted):
                property_names.extend(son)



from pypixiv.illust.item import ExtraData, Tags, Urls
from typing import Any, Optional


class DefaultIllustInfo:
    def __init__(self, response: Any) -> None:
        self._response = response

    def __repr__(self) -> str:
        return f"<title={self.title}>"

    @property
    def alt(self) -> str:
        return self._response["alt"]

    @property
    def bookMarkData(self) -> str:
        return self._response["bookmarkData"]

    @property
    def createDate(self) -> str:
        return self._response["createDate"]

    @property
    def description(self) -> str:
        return self._response["description"]

    @property
    def height(self) -> int:
        return self._response["height"]

    @property
    def id(self) -> str:
        return self._response["id"]

    @property
    def illustType(self) -> int:
        return self._response["illustType"]

    @property
    def isBookmarkable(self) -> str:
        return self._response["isBookmarkable"]

    @property
    def isUnlisted(self) -> str:
        return self._response["isUnlisted"]

    @property
    def title(self) -> str:
        return self._response["title"]

    @property
    def titleCaptionTranslation(self) -> str:
        return self._response["titleCaptionTranslation"]

    @property
    def width(self) -> int:
        return self._response["width"]

    @property
    def xRestrict(self) -> int:
        return self._response["xRestrict"]

    @property
    def pageCount(self) -> int:
        return self._response["pageCount"]

    @property
    def restrict(self) -> int:
        return self._response["restrict"]

    @property
    def sl(self) -> int:
        return self._response["sl"]


class UserIllustInfo(DefaultIllustInfo):
    def __init__(self, response: Any) -> None:
        super().__init__(response)

    @property
    def url(self) -> str:
        return self._response["url"]

    @property
    def profileImageUrl(self) -> str:
        return self._response["profileImageUrl"]

    @property
    def isMasked(self) -> Optional[str]:
        return self._response["isMasked"]

    @property
    def tags(self) -> list[str]:
        return self._response["tags"]

    @property
    def updateDate(self) -> str:
        return self._response["updateDate"]


class IllustBody(DefaultIllustInfo):
    def __init__(self, response: Any) -> None:
        super().__init__(response)

    @property
    def bookmarkCount(self) -> int:
        return self._response["bookmarkCount"]

    @property
    def comicPromotion(self) -> bool:
        return self._response["comicPromotion"]

    @property
    def commentCount(self) -> int:
        return self._response["commentCount"]

    @property
    def contestBanners(self) -> list[str]:
        return self._response["contestBanners"]

    @property
    def contestData(self) -> Optional[str]:
        return self._response["contestData"]

    @property
    def descriptionBoothId(self) -> Optional[str]:
        return self._response["descriptionBoothId"]

    @property
    def descriptionYoutubeId(self) -> Optional[str]:
        return self._response["descriptionYoutubeId"]

    @property
    def extraData(self) -> Optional[ExtraData]:
        if "extraData" in self._response:
            return ExtraData(self._response["extraData"])
        return

    @property
    def fanboxPromotion(self) -> Any:
        return self._response["fanboxPromotion"]

    @property
    def illustComment(self) -> str:
        return self._response["illustComment"]

    @property
    def illustTitle(self) -> str:
        return self._response["illustTitle"]

    @property
    def imageResponseCount(self) -> int:
        return self._response["imageResponseCount"]

    @property
    def imageResponseData(self) -> Any:
        return self._response["imageResponseData"]

    @property
    def imageResponseOutData(self) -> Any:
        return self._response["imageResponseOutData"]

    @property
    def isHowto(self) -> bool:
        return self._response["isHowto"]

    @property
    def isOriginal(self) -> bool:
        return self._response["isOriginal"]

    @property
    def likeCount(self) -> int:
        return self._response["likeCount"]

    @property
    def likeData(self) -> Any:
        return self._response["likeData"]

    @property
    def pollData(self) -> Optional[Any]:
        return self._response["pollData"]

    @property
    def request(self) -> Any:
        return self._response["request"]

    @property
    def responseCount(self) -> int:
        return self._response["responseCount"]

    @property
    def seriesNavData(self) -> Optional[Any]:
        return self._response["seriesNavData"]

    @property
    def storableTags(self) -> list[str]:
        return self._response["storableTags"]

    @property
    def tags(self) -> Tags:
        return Tags(self._response["tags"])

    @property
    def urls(self) -> Urls:
        return Urls(self._response["urls"])

    @property
    def userAccount(self) -> str:
        return self._response["userAccount"]

    @property
    def viewCount(self) -> int:
        return self._response["viewCount"]



import setuptools  # type: ignore
from setuptools import setup  # type: ignore

import pypixiv

setup(
    name="pypixiv",
    author="Ryu JuHeon",
    author_email="SaidBySolo@gmail.com",
    url="https://github.com/SaidBySolo/pypixiv",
    version=pypixiv.__version__,
    long_description=open("README.md", "rt", encoding="UTF8").read(),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),  # type: ignore
    description="Pixiv ajax wrapper",
    python_requires=">=3.9",
)



from __future__ import annotations
from pypixiv.abc import BasePixiv

from pypixiv.illust.item import ZengoWorkData
from pypixiv.illust.body import IllustBody, UserIllustInfo
from typing import Any


class NoLoginData:
    def __init__(self, item: dict[str, Any]):
        self._item = item

    # beadcrumbs

    @property
    def zengoIdWorks(self) -> list[UserIllustInfo]:
        return [UserIllustInfo(item) for item in self._item["zengoIdWorks"]]

    @property
    def zengoWorkData(self) -> ZengoWorkData:
        return ZengoWorkData(self._item["zengoWorkData"])


class IllustInfo(BasePixiv):
    def __init__(self, response: dict[str, Any]):
        super().__init__(response)

    @property
    def body(self) -> IllustBody:
        return IllustBody(super().body)



from __future__ import annotations

from typing import Any

class BasePixiv:
    # error: str
    # message: str
    # body: dict[str, Any] | list[dict[str, Any]]

    def __init__(self, response: Any) -> None:
        self.__response = response

    @property
    def error(self) -> str:
        return self.__response["error"]

    @property
    def message(self) -> str:
        return self.__response["message"]

    @property
    def body(self) -> Any:
        return self.__response["body"]


class BaseRanking:
    def __init__(self, response: Any) -> None:
        self.__response = response

    @property
    def contents(self) -> list[dict[str, Any]]:
        return self.__response["contents"]

    @property
    def mode(self) -> str:
        return self.__response["mode"]

    @property
    def content(self) -> str:
        return self.__response["content"]

    @property
    def page(self) -> int:
        return self.__response["page"]

    @property
    def prev(self) -> bool:
        return self.__response["prev"]

    @property
    def next(self) -> int:
        return self.__response["next"]

    @property
    def date(self) -> str:
        return self.__response["date"]

    @property
    def prev_date(self) -> str:
        return self.__response["prev_date"]

    @property
    def next_date(self) -> str | bool:
        return self.__response["next_date"]

    @property
    def rank_total(self) -> int:
        return self.__response["rank_total"]



from typing import Literal, Optional

from aiohttp.client import ClientSession

from pypixiv.http import PixivHttpClient
from pypixiv.illust.model import IllustInfo
from pypixiv.user.model import UserFullInfo, UserInfo
from pypixiv.ranking.model import RankingInfo


class PixivClient(PixivHttpClient):
    """
    Pixiv API client class.
    """

    def __init__(
        self, session: Optional[ClientSession] = None, lang: str = "ko"
    ) -> None:
        super().__init__(session=session, lang=lang)

    async def userinfo(self, user_id: int):
        return UserInfo(await self.get_userinfo(user_id))

    async def full_userinfo(self, user_id: int):
        return UserFullInfo(await self.get_full_userinfo(user_id))

    async def illustinfo(self, illust_id: int):
        return IllustInfo(await self.get_illust(illust_id))

    async def ranking(
        self, mode: Literal["daily", "weekly", "monthly"], date: Optional[str]
    ):
        return RankingInfo(await self.get_ranking(mode, date))



from typing import Any, Optional


class Privacy:
    def __init__(self, item: Any):
        self.__item = item

    @property
    def name(self) -> Optional[str]:
        return self.__item["name"]

    @property
    def privacyLeve(self) -> Optional[bool]:
        return self.__item["privacyLevel"]


class SketchLives:
    def __init__(self, item: Any):
        self.__item = item

    @property
    def audienceCount(self) -> Optional[int]:
        return self.__item["audienceCount"]

    @property
    def id(self) -> Optional[str]:
        return self.__item["id"]

    @property
    def isR18(self) -> Optional[bool]:
        return self.__item["isR18"]

    @property
    def name(self) -> Optional[str]:
        return self.__item["name"]

    @property
    def streamerIds(self) -> Optional[list[int]]:
        return self.__item["streamerIds"]

    @property
    def thumbnailUrl(self) -> Optional[str]:
        return self.__item["thumbnailUrl"]

    @property
    def url(self) -> Optional[str]:
        return self.__item["url"]


class Group:
    def __init__(self, item: Any):
        self.__item = item

    @property
    def iconUrl(self) -> Optional[str]:
        return self.__item["iconUrl"]

    @property
    def id(self) -> Optional[str]:
        return self.__item["id"]

    @property
    def title(self) -> Optional[str]:
        return self.__item["title"]


class SocialUrl:
    def __init__(self, item: Any):
        self.__item = item

    @property
    def url(self) -> Optional[str]:
        return self.__item["url"]


class Pawoo(SocialUrl):
    def __init__(self, item: Any):
        super().__init__(item)


class Twitter(SocialUrl):
    def __init__(self, item: Any):
        super().__init__(item)


class Social:
    def __init__(self, item: Any):
        self.__item = item

    @property
    def pawoo(self) -> Optional[Pawoo]:
        if "pawoo" in self.__item:
            return Pawoo(self.__item["pawoo"])

    @property
    def twitter(self) -> Optional[Twitter]:
        if "twitter" in self.__item:
            return Twitter(self.__item["twitter"])


class WorkSpace:
    def __init__(self, item: Any):
        self.__item = item

    @property
    def userWorkspaceChair(self) -> Optional[str]:
        return self.__item.get("userWorkspaceChair")

    @property
    def userWorkspaceComment(self) -> Optional[str]:
        return self.__item.get("userWorkspaceComment")

    @property
    def userWorkspaceDesk(self) -> Optional[str]:
        return self.__item.get("userWorkspaceDesk")

    @property
    def userWorkspaceDesktop(self) -> Optional[str]:
        return self.__item.get("userWorkspaceDesktop")

    @property
    def userWorkspaceMonitor(self) -> Optional[str]:
        return self.__item.get("userWorkspaceMonitor")

    @property
    def userWorkspaceMouse(self) -> Optional[str]:
        return self.__item.get("userWorkspaceMouse")

    @property
    def userWorkspacePc(self) -> Optional[str]:
        return self.__item.get("userWorkspacePc")

    @property
    def userWorkspacePrinter(self) -> Optional[str]:
        return self.__item.get("userWorkspacePrinter")

    @property
    def userWorkspaceScanner(self) -> Optional[str]:
        return self.__item.get("userWorkspaceScanner")

    @property
    def userWorkspaceTablet(self) -> Optional[str]:
        return self.__item.get("userWorkspaceTablet")

    @property
    def userWorkspaceTool(self) -> Optional[str]:
        return self.__item.get("userWorkspaceTool")

    @property
    def wsBigUrl(self) -> Optional[str]:
        return self.__item.get("wsBigUrl")

    @property
    def wsUrl(self) -> Optional[str]:
        return self.__item.get("wsUrl")
